#!/bin/bash
# EIBD eib bus access and management daemon
# Copyright (C) 2005-2006 Martin Kgler <mkoegler@auto.tuwien.ac.at>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

DIALOG= 
TMP=`mktemp`
trap 'rm -f $TMP' EXIT

if [ "$DISPLAY" != "" ]; then
    DIALOG=`which Xdialog`
    DLGOPT=--wrap
fi
if [ ! -x "$DIALOG" ]; then
    DIALOG=`which dialog`
    DLGOPT=
fi

if [ ! -x $DIALOG ]; then
    echo $"No dialog program found. Please install Xdialog or dialog."
    exit 1
fi

DIALOG="$DIALOG $DLGOPT"

function usage()
{
    echo "$0 [-b (usb|tpuart|bcu1|bcu2|eibnettunnel|eibnetrouting)] [-k] [-d confdir] [-s sucmd] [-i on/off] [-r on/off] [-t on/off] [-R]" 
    echo $" EIBD configuration wizard"
    echo $"Copyright (C) 2005-2006 Martin Kgler <mkoegler@auto.tuwien.ac.at>"
    echo
    echo $" -k               Stop EIBD instance started by this script"
    echo $" -R               Write address table length"
    echo $" -a               Do not change BCU address table size"
    echo $" -q               Be quiet"
    echo $" -i on/off        Export EIBD Protocol to the network"
    echo $" -r on/off        Start EIBnet/IP Routing Server"
    echo $" -t on/off        Start EIBnet/IP Tunneling Server"
    echo $" -s SUCMD         Use SUCMD to execute commands as root (default sudo)"
    echo $" -b usb           Connect via USB"
    echo $" -b tpuart        Connect via a TPUART" 
    echo $" -b bcu1          Connect via a BCU 1"
    echo $" -b bcu2          Connect via a BCU 2"
    echo $" -b eibnettunnel  Connect via a EIBnet/IP Router"
    echo $" -b eibnetrouting Connect via a EIBnet/IP Router"
    echo $" -d <directory>   Directory for status files"
    echo $" -p               Print PID file name"
}

MODE=start
BACKEND=
DIR="/tmp/eibd-$USER"
ADDR=1
SU=`which sudo`
EIBDI=
EIBDR=
EIBDT=
URL=
DEVOPT=
QUIET=
if [ ! -x "$SU" ]; then
    SU=`which su`
fi

function menucmd()
{
    printf "%s --title '%s' --menu '%s' 24 48 15 %s 2>&1 1>&3" "$DIALOG" "$1" "$2" "$3"
}

function gauge()
{
    $DIALOG --title "$1" --gauge "$2" 24 48 
}

while getopts "khb:d:as:r:t:i:Rpq" option; do
    case "$option" in
	k ) MODE=stop ;;
	R ) MODE=reset ;;
	p ) MODE=pid ;;
	b ) BACKEND="$OPTARG" ;;
	d ) DIR="$OPTARG" ;;
	a ) ADDR=0 ;;
	q ) QUIET=1 ;;
	s ) SU="$OPTARG" ;;
	r ) EIBDR="$OPTARG" ;;
	t ) EIBDT="$OPTARG" ;;
	i ) EIBDI="$OPTARG" ;;
	* ) usage
	    exit 1 ;;
    esac
done

if [ "$MODE" == "pid" ]; then
    echo "$DIR/eibd.pid"
    exit 0
fi

if [ "$MODE" == "stop" ]; then
    if [ "$QUIET" == 1 ]; then
	DIALOG=/bin/echo
	DLGOPT=
    fi
    if [ ! -d "$DIR" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Directory with status files not found. Aborting." 8 45
	exit 1
    fi
    if [ -f "$DIR/eibd.pid" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Stopping EIBD" 8 45 || exit 1
	kill -SIGTERM "`cat $DIR/eibd.pid`"
	sleep 1
    fi
    for a in 1 2 3 4 5 6;
    do
      if [ ! -f "$DIR/eibd.pid" ]; then
	  break
      fi
      sleep 10
    done
    if [ -f "$DIR/eibd.pid" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Shutdown of eibd failed. It will be killed." 8 45

	kill -SIGKILL "`cat $DIR/eibd.pid`"
	sleep 5
	rm "$DIR/eibd.pid"
    fi
    if [ "$ADDR" == 1 ]; then
	if [ -f "$DIR/addrtab" ]; then
	    if bcuaddrtab -T 60 -w "`cat "$DIR/addrtab"`" "`cat "$DIR/url"`" ; then
		echo $"Address table length changed successfully"
		rm "$DIR/addrtab"
	    else
		$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Reset of address table length failed. Exiting." 8 45
	    fi
	fi
    fi
    if [ "`cat $DIR/url|cut -d: -f1`" == "bcu1" ]; then
	$SU setserial /dev/ttyS0 autoconfig
	$SU setserial /dev/ttyS1 autoconfig
	$SU setserial /dev/ttyS2 autoconfig
	$SU setserial /dev/ttyS3 autoconfig
    fi
    exit 0
fi

if mkdir -p "$DIR" ; then
    true
else
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Creation of status directory failed. Aborting." 8 45
    exit 1
fi

if [ -f "$DIR/eibd.pid" ]; then
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"EIBD is already running. It will be stopped." 8 45 || exit 1
    $0 -k
fi

exec 3>&1

if [ "$BACKEND" != "usb" -a "$BACKEND" != "bcu1" -a "$BACKEND" != "bcu2" -a "$BACKEND" != "tpuart" -a "$BACKEND" != "eibnetrouting" -a "$BACKEND" != "eibnettunnel" ]; then
    if [ "$MODE" == "reset" ]; then
	CMD=`menucmd  $"EIBD Connectivity Setup" $"Select the type of interface you use:" \
	    '"usb" $"KNX USB Interface" \
	    "bcu1" $"BCU 1" \
	    "bcu2" $"BCU 2" ' `
    else
	CMD=`menucmd  $"EIBD Connectivity Setup" $"Select the type of interface you use:" \
	    '"usb" $"KNX USB Interface" \
	    "bcu1" $"BCU 1" \
	    "bcu2" $"BCU 2" \
	    "tpuart" $"TPUART" \
	    "eibnettunnel" $"EIBnet/IP Tunneling" \
	    "eibnetrouting" $"EIBnet/IP Routing (not supported by all EIBnet/IP interfaces)"' `
    fi
    BACKEND=`eval $CMD`|| exit 1
fi

if [ "$BACKEND" == "usb" ]; then
    DEVICES=
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The setup wizard will now search for available devices. This may take up to one minute. Select OK to start the scan." 8 45 || exit 1
    $SU `which findknxusb` > $TMP
    while read c a b ; do
	if [ "$c" != "device" ]; then
	    continue
	fi
	DEVICES="$DEVICES '$a' '$b'"
    done < $TMP
    if [ "$DEVICES" == "" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No KNX USB interface found" 8 45
	exit 1
    fi
    CMD=`menucmd $"EIBD Connectivity Setup" $"Select the KNX USB interface you want to use:" "$DEVICES"`
    DEVICE=`eval $CMD` || exit 1

    USBFS=`cat /proc/mounts|grep " usbfs "|cut -d' ' -f2`
    FILE=`printf "$USBFS/%03d/%03d" \`echo "$DEVICE"|cut -d: -f1\` \`echo "$DEVICE"|cut -d: -f2\``

    $SU chmod 777 "$FILE"
    URL="usb:$DEVICE"
fi

if [ "$BACKEND" == "bcu2" ]; then
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The setup wizard will now search for available devices. This may take up to one minute. Select OK to start the scan." 8 45 || exit 1
    DEVICES=
    if [ -c /dev/ttyS0 ]; then
	$SU chmod 777 /dev/ttyS0
	$SU setserial /dev/ttyS0 autoconfig
	if bcuaddrtab -T 10 ft12:/dev/ttyS0 >/dev/null ; then
	    DEVICES="$DEVICES /dev/ttyS0 '(COM 1)'"
	fi
    fi
    if [ -c /dev/ttyS1 ]; then
	$SU chmod 777 /dev/ttyS1
	$SU setserial /dev/ttyS1 autoconfig
	if bcuaddrtab -T 10 ft12:/dev/ttyS1 >/dev/null ; then
	    DEVICES="$DEVICES /dev/ttyS1 '(COM 2)'"
	fi
    fi
    if [ -c /dev/ttyS2 ]; then
	$SU chmod 777 /dev/ttyS2
	$SU setserial /dev/ttyS2 autoconfig
	if bcuaddrtab -T 10 ft12:/dev/ttyS2 >/dev/null ; then
	    DEVICES="$DEVICES /dev/ttyS2 '(COM 3)'"
	fi
    fi
    if [ -c /dev/ttyS3 ]; then
	$SU chmod 777 /dev/ttyS3
	$SU setserial /dev/ttyS3 autoconfig
	if bcuaddrtab -T 10 ft12:/dev/ttyS3 >/dev/null ; then
	    DEVICES="$DEVICES /dev/ttyS3 '(COM 4)'"
	fi
    fi
    if [ "$DEVICES" == "" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No BCU 2 interface found. Aborting." 8 45
	exit 1
    fi

    CMD=`menucmd $"EIBD Connectivity Setup" $"Select the BCU 2 interface you want to use:" "$DEVICES"`
    DEVICE=`eval $CMD` || exit 1
    URL="ft12:$DEVICE"
fi

if [ "$BACKEND" == "bcu1" ]; then
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The setup wizard will now search for available devices. This may take up to one minute. Select OK to start the scan." 8 45 || exit 1
    $SU modprobe eib.ko
    sleep 5
    if grep " eib" /proc/devices ; then
    	true;
    else
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No BCU 1 driver loaded. Aborting." 8 45
	exit 1
    fi	
    DEVICES=
    if [ -c /dev/eib0 ]; then
	$SU chmod 777 /dev/eib0
	$SU setserial /dev/ttyS0 uart none
	if bcuaddrtab -T 10 bcu1:/dev/eib0 >/dev/null ; then
	    DEVICES="$DEVICES /dev/eib0 '(COM 1, /dev/ttyS0)'"
	fi
    fi
    if [ -c /dev/eib1 ]; then
	$SU chmod 777 /dev/eib1
	$SU setserial /dev/ttyS1 uart none
	if bcuaddrtab -T 10 bcu1:/dev/eib1 >/dev/null ; then
	    DEVICES="$DEVICES /dev/eib1 '(COM 2, /dev/ttyS1)'"
	fi
    fi
    if [ -c /dev/eib2 ]; then
	$SU chmod 777 /dev/eib2
	$SU setserial /dev/ttyS2 uart none
	if bcuaddrtab -T 10 bcu1:/dev/eib2 >/dev/null ; then
	    DEVICES="$DEVICES /dev/eib2 '(COM 3, /dev/ttyS2)'"
	fi
    fi
    if [ -c /dev/eib3 ]; then
	$SU chmod 777 /dev/eib3
	$SU setserial /dev/ttyS3 uart none
	if bcuaddrtab -T 10 bcu1:/dev/eib3 >/dev/null ; then
	    DEVICES="$DEVICES /dev/eib3 '(COM 4, /dev/ttyS3)'"
	fi
    fi
    if [ "$DEVICES" == "" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No BCU 1 Interface found. Aborting." 8 45
	exit 1
    fi

    CMD=`menucmd $"EIBD Connectivity Setup" $"Select the BCU 1 interface, you want to use:" "$DEVICES"`
    DEVICE=`eval $CMD` || exit 1
    URL="bcu1:$DEVICE"
fi

if [ "$BACKEND" == "tpuart" ]; then
    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The setup wizard will now search for available devices. This may take up to one minute. Select OK to start the scan." 8 45 || exit 1
    DEVICES=
    if [ -c /dev/ttyS0 ]; then
	$SU chmod 777 /dev/ttyS0
	$SU setserial /dev/ttyS0 autoconfig
	if true ; then
	    DEVICES="$DEVICES /dev/ttyS0 '(COM 1)'"
	fi
    fi
    if [ -c /dev/ttyS1 ]; then
	$SU chmod 777 /dev/ttyS1
	$SU setserial /dev/ttyS1 autoconfig
	if true ; then
	    DEVICES="$DEVICES /dev/ttyS1 '(COM 2)'"
	fi
    fi
    if [ -c /dev/ttyS2 ]; then
	$SU chmod 777 /dev/ttyS2
	$SU setserial /dev/ttyS2 autoconfig
	if true ; then
	    DEVICES="$DEVICES /dev/ttyS2 '(COM 3)'"
	fi
    fi
    if [ -c /dev/ttyS3 ]; then
	$SU chmod 777 /dev/ttyS3
	$SU setserial /dev/ttyS3 autoconfig
	if true ; then
	    DEVICES="$DEVICES /dev/ttyS3 '(COM 4)'"
	fi
    fi
    if [ "$DEVICES" == "" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No TPUART Interface found. Aborting." 8 45
	exit 1
    fi

    CMD=`menucmd $"EIBD Connectivity Setup" $"Select the TPUART interface you want to use:" "$DEVICES"`
    DEVICE=`eval $CMD` || exit 1
    URL="tpuarts:$DEVICE"
fi

if [ "$BACKEND" == "eibnetrouting" ]; then
    URL="ip:"
    if /sbin/ip route get 224.0.23.12; then
	true
    else
	$DIALOG --title "Network setup" --msgbox "You have no route for the Multicast addresses. A local route will be added for them."  10 41 || exit 1
	sudo /sbin/route add default lo
    fi
fi

if [ "$BACKEND" == "eibnettunnel" ]; then
    export DEVICES=""
    NAME=""
    IP=""
    OK=""
    if /sbin/ip route get 224.0.23.12; then
	true
    else
	$DIALOG --title "Network setup" --msgbox "You have no route for the Multicast addresses. A local route will be added for them."  10 41 || exit 1
	sudo /sbin/route add default lo
    fi

    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The setup wizard will now search for available devices. This may take up to one minute. Select OK to start the scan." 8 45 || exit 1
    eibnetsearch - > $TMP
    while read a b; do
	if [ "$a" == "Answer" ]; then
	    if [ "$OK" == 1 ]; then
		DEVICES="$DEVICES '$IP' '$NAME'"
	    fi
	    NAME=""
	    OK=""
	    IP="`echo "$b"|cut -d' ' -f2`:`echo "$b"|cut -d' ' -f5`"
	elif [ "$a" == "Name:" ]; then
	    NAME="$b"
	elif [ "$a" == "Service" ]; then
	    if [ "`echo "$b"|cut -d' ' -f1`" == "4" ]; then
		OK=1
	    fi
	fi
    done < $TMP

    if [ "$OK" == 1 ]; then
	DEVICES="$DEVICES '$IP' '$NAME'"
    fi

    if [ "$DEVICES" == "" ]; then
	$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"No EIBnet/IP Interface found. Aborting." 8 45
	exit 1
    fi

    CMD=`menucmd $"EIBD Connectivity Setup" $"Select the EIBnet/IP interface you want to use:" "$DEVICES"`
    DEVICE=`eval $CMD` || exit 1
    URL="ipt:$DEVICE"

fi

echo "$URL" > "$DIR/url"

if [ "$MODE" == reset ]; then
    if [ "$BACKEND" == "usb" -o "$BACKEND" == "bcu1" -o "$BACKEND" == "bcu2" ]; then
	SIZE=`$DIALOG --title $"EIBD Connectivity Setup" --inputbox \
	    $"Enter the address table size:" 14 48 0 \
	    2>&1 1>&3 ` || exit 1
	
	if bcuaddrtab  -T 60 -w "$SIZE" "$URL" ; then
	    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Address table size changed." 8 45
	    exit 0
	else
	    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The change of the address table size failed. Aborting." 8 45
	    exit 1
	fi
    fi
    exit 0;
fi

if [ "$BACKEND" == "usb" -o "$BACKEND" == "bcu1" -o "$BACKEND" == "bcu2" ]; then
    if [ "$ADDR" == 1 ]; then
	if bcuaddrtab  -T 30 $URL > $TMP ; then
	    true
	else
	    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"The read of the address table size failed. Aborting." 8 45
	    exit 1
	fi
	SIZE=`cat $TMP|grep "Size:" | cut -d: -f2`

	$DIALOG --title $"EIBD Connectivity Setup" --msgbox \
	    $"The address table length of your interface is $SIZE. Please make a note of this value in case you need to restore it manually." 8 45 || exit 1

	if bcuaddrtab -T 30 -w 0 $URL ; then
	    echo "$SIZE" > "$DIR/addrtab"
	else
	    $DIALOG --title $"EIBD Connectivity Setup" --msgbox $"Changing of the address table size failed. Aborting." 8 45
	    exit 1
	fi
    fi
fi

if [ "$BACKEND" == tpuart -o "$BACKEND" == eibnetrouting ]; then
	BCUADDR=`$DIALOG --title $"EIBD Connectivity Setup" --inputbox \
	    $"Enter the individual (physical) address that shall be assigned to the interface:" 24 48 "0.0.1" \
	    2>&1 1>&3 ` || exit 1
    DEVOPT="-e '$BCUADDR'"
fi

EIBDOPT="-u "

if [ "$EIBDI" != on -a "$EIBDI" != off ]; then
    EIBDI=on
    $DIALOG --title $"EIBD Connectivity Setup" --yesno $"Do you want to expose the EIBD protocol to the network?" 8 45 || EIBDI=off
fi

if [ "$EIBDT" != on -a "$EIBDT" != off ]; then
    EIBDT=on
    $DIALOG --title $"EIBD Connectivity Setup" --yesno $"Do you want to start the EIBnet/IP Tunneling server?" 8 45 || EIBDT=off
fi

if [ "$EIBDR" != on -a "$EIBDR" != off ]; then
    EIBDR=on
    $DIALOG --title $"EIBD Connectivity Setup" --yesno $"Do you want to start the EIBnet/IP Routing server?" 8 45 || EIBDR=off
fi

if [ "$EIBDI" == "on" ]; then
    EIBDOPT="$EIBDOPT -i"
fi

if [ "$EIBDT" == "on" -o "$EIBDR" == "on" ]; then
    EIBDOPT="$EIBDOPT -DS"
    if /sbin/ip route get 224.0.23.12; then
	true
    else
	$DIALOG --title "Network setup" --msgbox "You have no route for the Multicast addresses. A local route will be added for them."  10 41 || exit 1
	sudo /sbin/route add default lo
    fi
fi

if [ "$EIBDT" == "on" ]; then
    EIBDOPT="$EIBDOPT -T"
fi

if [ "$EIBDR" == "on" ]; then
    EIBDOPT="$EIBDOPT -R"
fi

setsid eibd -d -p "$DIR/eibd.pid" $EIBDOPT $DEVOPT $URL 

$DIALOG --title $"EIBD Connectivity Setup" --msgbox $"EIBD started." 8 45
